@@ -9,7 +9,6 @@ class CustomField(FeatureBase):
     """
     This feature provides high-level interaction with TPP Custom Field objects.
     """
-
     def __init__(self, api):
         super().__init__(api)
         self._metadata_root_dn = r'\VED\Metadata Root'
@@ -32,8 +31,8 @@ class CustomField(FeatureBase):
                localized_help: Union[List[str], Dict] = None, localized_label: str = None, localized_set: str = None,
                mandatory: bool = None, mask: str = None, maximum_length: int = None, minimum_length: int = None,
                policyable: bool = None, regular_expression: str = None, render_hidden: bool = None,
-               render_read_only: bool = None, single: bool = None, time_only: bool = None
-               ):
+               render_read_only: bool = None, single: bool = None, time_only: bool = None,
+               get_if_already_exists: bool = True):
         """
         Creates a Custom Field (or Metadata Item). Custom fields appear in the UI unless ``render_hidden = True``. If
         the custom field should be implemented on a folder policy, then set ``policyable=True``, otherwise the custom
@@ -98,6 +97,7 @@ class CustomField(FeatureBase):
             render_read_only: If ``False``, the user can change the default values (if not locked by policy).
             single: If ``False``, the user can select/input multiple items.
             time_only: If ``True``, time of day is required.
+            get_if_already_exists: If the objects already exists, just return it as is.
 
         Returns:
             Metadata Item object.
@@ -133,21 +133,24 @@ class CustomField(FeatureBase):
         }
 
         response = self._api.websdk.Metadata.DefineItem.post(item=item)
+        if response.result.code == 9 and get_if_already_exists:
+            return self.get_item_details(custom_field=name)
         self._validate_result_code(response.result)
         return response.item
 
-    def delete(self, custom_field: 'Config.Object', remove_data: bool = True):
+    def delete(self, custom_field: Union['Config.Object', str], remove_data: bool = True):
         """
         Deletes a custom field and all instances of it, including policy settings. If ``remove_data = False``, then
         an exception will be raised if there is existing data for the custom field on any object.
 
         Args:
-            custom_field: Config object of the custom field
+            custom_field: Config.Object or name of the custom field
             remove_data: If ``True``, then deletes the custom field regardless of data existing on the custom field
                 on any object that implements it.
         """
+        custom_field_guid = self._get_guid(custom_field, parent_dn=self._metadata_root_dn)
         response = self._api.websdk.Metadata.UndefineItem.post(
-            item_guid=custom_field.guid,
+            item_guid=custom_field_guid,
             remove_data=remove_data
         )
         self._validate_result_code(response.result)
@@ -162,23 +165,21 @@ class CustomField(FeatureBase):
         Returns:
             Config object of the custom field.
         """
-        response = self._api.websdk.Config.IsValid.post(
-            object_dn=f'{self._metadata_root_dn}\\{name}'
-        )
-        return response.object
+        return self._get_config_object(object_dn=f'{self._metadata_root_dn}\\{name}')
 
-    def get_item_details(self, custom_field: 'Config.Object' = None):
+    def get_item_details(self, custom_field: Union['Config.Object', str] = None):
         """
         Fetches the item details of a custom field object according to the data
         returned by the WebSDK API "POST Metadata/LoadItem".
 
         Args:
-            custom_field: The name of the custom field object.
+            custom_field: The Config.Object or name of the custom field object.
 
         Returns:
             Metadata Item object.
         """
-        response = self._api.websdk.Metadata.LoadItem.post(dn=custom_field.dn)
+        custom_field_dn = self._get_dn(custom_field, parent_dn=self._metadata_root_dn)
+        response = self._api.websdk.Metadata.LoadItem.post(dn=custom_field_dn)
         self._validate_result_code(result=response.result)
         return response.item
 
@@ -193,13 +194,13 @@ class CustomField(FeatureBase):
         self._validate_result_code(response.result)
         return response.items
 
-    def read(self, obj: 'Config.Object', custom_field: 'Config.Object'):
+    def read(self, obj: Union['Config.Object', str], custom_field: Union['Config.Object', str]):
         """
         Reads the actual value(s) of a custom field, accounting for policy settings. Value(s) may be None.
 
         Args:
-            obj: Config object of the object having the custom field.
-            custom_field: Config object of the custom field.
+            obj: Config object or DN of the object having the custom field.
+            custom_field: Config.Object or name of the custom field.
 
         Returns:
             EffectiveValues object:
@@ -207,9 +208,11 @@ class CustomField(FeatureBase):
                 * policy_dn: Absolute path to the policy locking the values.
                 * values: List of values.
         """
+        obj_dn = self._get_dn(obj)
+        custom_field_guid = self._get_guid(custom_field, parent_dn=self._metadata_root_dn)
         response = self._api.websdk.Metadata.ReadEffectiveValues.post(
-            dn=obj.dn,
-            item_guid=custom_field.guid
+            dn=obj_dn,
+            item_guid=custom_field_guid
         )
         self._validate_result_code(response.result)
 
@@ -221,13 +224,13 @@ class CustomField(FeatureBase):
 
         return EffectiveValues()
 
-    def read_policy(self, folder: 'Config.Object', custom_field: 'Config.Object', class_name: str):
+    def read_policy(self, folder: Union['Config.Object', str], custom_field: Union['Config.Object', str], class_name: str):
         """
         Reads the policy value(s) of a custom field, accounting for policy settings. Value(s) may be None.
 
         Args:
-            folder: Config object of the folder.
-            custom_field: Config object of the custom field object.
+            folder: Config object or name of the folder.
+            custom_field: Config.Object or name of the custom field object.
             class_name: Object class.
 
         Returns:
@@ -235,9 +238,11 @@ class CustomField(FeatureBase):
                 * locked: Boolean. If ``True``, the ``values`` are locked by the policy.
                 * values: List of values.
         """
+        folder_dn = self._get_dn(folder)
+        custom_field_guid = self._get_guid(custom_field, parent_dn=self._metadata_root_dn)
         response = self._api.websdk.Metadata.ReadPolicy.post(
-            dn=folder.dn,
-            item_guid=custom_field.guid,
+            dn=folder_dn,
+            item_guid=custom_field_guid,
             obj_type=class_name
         )
         self._validate_result_code(response.result)
@@ -249,7 +254,7 @@ class CustomField(FeatureBase):
 
         return PolicyValues()
 
-    def update(self, custom_field: 'Config.Object', allowed_characters: List[str] = None, allowed_values: List[str] = None,
+    def update(self, custom_field: Union['Config.Object', str], allowed_characters: List[str] = None, allowed_values: List[str] = None,
                category: str = None, classes: list = None, data_type: int = None, date_only: bool = None,
                default_values: str = None, display_after: str = None, error_message: str = None, help_text: str = None,
                label: str = None, localization_table: str = None, localized_help: str = None, localized_label: str = None,
@@ -261,7 +266,7 @@ class CustomField(FeatureBase):
         property. Property values are not reset to their defaults.
 
         Args:
-            custom_field: Config object of the metadata object.
+            custom_field: Config object or name of the metadata object.
             label: The label text for the custom field.
             name: The name of the custom field object.
             classes: The classes that this field applies to.
@@ -291,6 +296,7 @@ class CustomField(FeatureBase):
         Returns:
             Metadata Item object.
         """
+        custom_field_dn = self._get_dn(custom_field, parent_dn=self._metadata_root_dn)
         listify = lambda x: [x] if x and not isinstance(x, list) else x
 
         item = {
@@ -323,32 +329,35 @@ class CustomField(FeatureBase):
         }
         response = self._api.websdk.Metadata.UpdateItem.post(
             update={
-                "DN": custom_field.dn,
+                "DN": custom_field_dn,
                 "Data": self._name_value_list(attributes=item, keep_list_values=True)
             }
         )
         self._validate_result_code(response.result)
         return self.get_item_details(custom_field=custom_field)
 
-    def write(self, obj: 'Config.Object', custom_field: 'Config.Object', values: List, keep_existing: bool = True):
+    def write(self, obj: Union['Config.Object', str], custom_field: Union['Config.Object', str], values: List,
+              keep_existing: bool = True):
         """
         Writes a set of values to a custom field on the specified ``obj``. If ``keep_existing = False``,
         then all custom fields related to the ``obj`` are reset to their default values.
 
         Args:
-            obj: Config object of the object with the custom field.
-            custom_field: Config object of the custom field object.
+            obj: Config object or name of the object with the custom field.
+            custom_field: Config.Object or name of the custom field object.
             values: List of values to write to ``object_dn``.
             keep_existing: If ``True``, all other custom fields remain unaffected.
         """
+        obj_dn = self._get_dn(obj)
+        custom_field_guid = self._get_guid(custom_field)
         response = self._api.websdk.Metadata.Set.post(
-            dn=obj.dn,
-            guid_data=self._guid_data_list({custom_field.guid: values}),
+            dn=obj_dn,
+            guid_data=self._guid_data_list({custom_field_guid: values}),
             keep_existing=keep_existing
         )
         self._validate_result_code(result=response.result)
 
-    def write_policy(self, folder: 'Config.Object', custom_field: 'Config.Object', class_name: str,
+    def write_policy(self, folder: Union['Config.Object', str], custom_field: Union['Config.Object', str], class_name: str,
                      values: List, locked: bool = False):
         """
         Writes a set of values to a custom field on the specified ``folder``. The custom field MUST be policyable.
@@ -357,15 +366,17 @@ class CustomField(FeatureBase):
 
         Args:
             folder: Config object of the folder.
-            custom_field: Config object of the custom field object.
+            custom_field: Config.Object or name of the custom field object.
             class_name: Name of the object class.
             values: List of values to write to ``obj``.
             locked: If ``True``, all subordinate objects inherit and cannot modify the ``values``.
         """
+        folder_dn = self._get_dn(folder)
+        custom_field_guid = self._get_guid(custom_field)
         response = self._api.websdk.Metadata.SetPolicy.post(
-            dn=folder.dn,
+            dn=folder_dn,
             config_class=class_name,
-            guid_data=self._guid_data_list({custom_field.guid: values}),
+            guid_data=self._guid_data_list({custom_field_guid: values}),
             locked=locked
         )
         self._validate_result_code(result=response.result)
